//////////////////////////////////////////////////
///////////////Normalmappingshader////////////////
///////////////- 4 Lights/////////////////////////
///////////////- with clipping////////////////////
//////////////////////////////////////////////////
////03.05.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////

#ifndef OS_CI_Normalmapping
#define OS_CI_Normalmapping


//Normalmappingshader for PS 2.0 supporting 4 lights
MATERIAL* OS_CI_Normalmapping_ps20_mat =
{
	event = OS_CI_Normalmapping_mtl_event;
	flags = ENABLE_VIEW;
	
	effect =
	"
		//Variables
		float4x4 matWorldViewProj;	
		float4x4 matWorld;
		float3x3 matTangent;
		
		float4 vecSunDir;
		float4 vecSunColor;
		float4 vecLightPos[8];
		float4 vecLightColor[8];
		
		float4 vecViewDir;
		float4 vecViewPos;
		
		float4 vecFog;
		float4 vecLight;
		
		float4 vecSkill41;
		float4 vecSkill13;
		float4 vecSkill17;
		
		
		//Textures
		texture entSkin1; //this is the color map
		texture entSkin2; //this is the normal map
		
		
		//Sampler
		sampler colorMap = sampler_state
		{
		   Texture = <entSkin1>;
		   MinFilter = Linear;
		   MagFilter = Linear;
		   MipFilter = Linear;   
		   AddressU  = wrap;
		   AddressV  = wrap;
		};
		
		sampler bumpMap = sampler_state
		{
		   Texture = <entSkin2>;
		   MinFilter = Linear;
		   MagFilter = Linear;
		   MipFilter = Linear;   
		   AddressU  = wrap;
		   AddressV  = wrap;
		};
		
		
		//Structs
		struct VS_TO_PS
		{
			float4 Pos  : POSITION;
			float Fog : FOG;
			
			float3 Tex : TEXCOORD0;
			
			float3 View : TEXCOORD1;
			
			float4 Light1 : TEXCOORD2;
			float4 Light2 : TEXCOORD3;
			float4 Light3 : TEXCOORD4;
			float4 Light4 : TEXCOORD5;
		};
		
		
		//Vertexshader
		VS_TO_PS VS(float4 inPos : POSITION,
						float2 inTex : TEXCOORD0,
						float3 inNormal : NORMAL,
						float3 inTangent : TEXCOORD2	)
		{
			VS_TO_PS Out = (VS_TO_PS)0;
			
			//Calculate the final vertexposition
			Out.Pos = mul(inPos, matWorldViewProj);
			
			//Get the vertexposition in worldspace
			float3 PosWorld = mul(inPos, matWorld);
			
			//Create matTangent
			matTangent[0] = mul(inTangent, matWorld);
			matTangent[1] = mul(cross(inTangent, inNormal), matWorld);
			matTangent[2] = mul(inNormal, matWorld);
			
			//Pass the TexCoords
			Out.Tex.xy = inTex;
			
			//Viewdir to tangent space
			Out.View = mul(matTangent, vecViewPos.xyz-PosWorld.xyz);
			
			float3 LightDir = 0;
			
			//Light1
			LightDir = vecLightPos[0]-PosWorld;
			Out.Light1.xyz = normalize(mul(matTangent,LightDir));
			Out.Light1.w = saturate(1.0f - length(LightDir/vecLightPos[0].w));
			
			//Light2
			LightDir = vecLightPos[1]-PosWorld;
			Out.Light2.xyz = normalize(mul(matTangent,normalize(LightDir)));
			Out.Light2.w = saturate(1.0f - length(LightDir/vecLightPos[1].w));
			
			//Light3
			LightDir = vecLightPos[2]-PosWorld;
			Out.Light3.xyz = normalize(mul(matTangent,normalize(LightDir)));
			Out.Light3.w = saturate(1.0f - length(LightDir/vecLightPos[2].w));
			
			//Light4
			LightDir = vecLightPos[3]-PosWorld;
			Out.Light4.xyz = normalize(mul(matTangent,normalize(LightDir)));
			Out.Light4.w = saturate(1.0f - length(LightDir/vecLightPos[3].w));
			
			//Fog
			Out.Fog = 1 - (distance(PosWorld,vecViewPos)-vecFog.x) * vecFog.z;
			
			//Clipping
			if(vecSkill17.w == 1.0)
			{
				Out.Tex.z = (PosWorld.y-vecSkill13.w)*vecSkill17.z;
			}
			
			return Out;
		}
		
		
		//Pixelshader
		float4 PS(VS_TO_PS In) : COLOR
		{
			//Clipping
			clip(In.Tex.z);
			
			//get the textures
			float4 Color = tex2D(colorMap,In.Tex.xy);
			float4 Bump = tex2D(bumpMap,In.Tex.xy);
			Bump.rgb = Bump.rgb * 2.0f - 1.0f;
			
			Bump.a *= vecSkill41.x;
			
			In.View = normalize(In.View);
			
			//Light1
			float3 Light1Diff = saturate(dot(Bump.rgb,In.Light1.xyz));
			float3 Light1Reflect = normalize(2 * Light1Diff * Bump.rgb - In.Light1.xyz);
			float3 Light1Spec = pow(saturate(dot(Light1Reflect,In.View)), vecSkill41.y)*Bump.a;
			float3 Light1 = (Color.rgb*Light1Diff+Light1Spec) * vecLightColor[0] * In.Light1.w;
			
			//Light2
			float3 Light2Diff = saturate(dot(Bump.rgb,In.Light2.xyz));
			float3 Light2Reflect = normalize(2 * Light2Diff * Bump.rgb - In.Light2.xyz);
			float3 Light2Spec = pow(saturate(dot(Light2Reflect,In.View)), vecSkill41.y)*Bump.a;
			float3 Light2 = (Color.rgb*Light2Diff+Light2Spec) * vecLightColor[1] * In.Light2.w;
			
			//Light3
			float3 Light3Diff = saturate(dot(Bump.rgb,In.Light3.xyz));
			float3 Light3Reflect = normalize(2 * Light3Diff * Bump.rgb - In.Light3.xyz);
			float3 Light3Spec = pow(saturate(dot(Light3Reflect,In.View)), vecSkill41.y)*Bump.a;
			float3 Light3 = (Color.rgb*Light3Diff+Light3Spec) * vecLightColor[2] * In.Light3.w;
			
			//Light4
			float3 Light4Diff = saturate(dot(Bump.rgb,In.Light4.xyz));
			float3 Light4Reflect = normalize(2 * Light4Diff * Bump.rgb - In.Light4.xyz);
			float3 Light4Spec = pow(saturate(dot(Light4Reflect,In.View)), vecSkill41.y)*Bump.a;
			float3 Light4 = (Color.rgb*Light4Diff+Light4Spec) * vecLightColor[3] * In.Light4.w;
			
			//Compute the final color
			Color.rgb = Color.rgb*vecLight.rgb+Light1+Light2+Light3+Light4;
			
			return Color;
		}
		
		
		//Technique
		technique NormalMapping
		{
			pass P0
			{
				alphatestenable = true;
				zWriteEnable = true;
				
				// compile shaders
				VertexShader = compile vs_1_1 VS();
				PixelShader  = compile ps_2_0 PS();
			}
		}
	";
}

void OS_CI_Normalmapping_mtl_event()
{
	mtl.skill20 = 0;
	
	mtl.skill14 = floatv(render_view.portal_x);
	mtl.skill15 = floatv(render_view.portal_y);
	mtl.skill16 = floatv(render_view.portal_z);
	
	mtl.skill17 = floatv(render_view.pnormal_x);
	mtl.skill18 = floatv(render_view.pnormal_y);
	mtl.skill19 = floatv(render_view.pnormal_z);
	
	#ifdef OS_Water
		if(render_view == OS_Water_Reflect_view || render_view == OS_Water_Refract_view)
		{
			mtl.skill20 = floatv(1);
		}
	#endif
}


//assigns the shader to the entity
void OS_CI_Normalmapping_init(ENTITY* Ent, var SpecStrength, var SpecIntensity)
{
	if(d3d_shaderversion >= 2020)
	{
		OS_Mtl_set(Ent,OS_I_Normalmapping_ps20_mat);
		
		Ent.skill41 = floatv(SpecStrength);
		Ent.skill42 = floatv(SpecIntensity);
	}
}

#endif