//////////////////////////////////////////////////
//////////////////Glow Shader/////////////////////
//////////////////////////////////////////////////
////07.05.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////

#ifndef OS_Glow
#define OS_Glow

//
MATERIAL* OS_Glow_ps20_mat =
{
	effect = "
	//Variables
		float4x4 matWorld;
		float4x4 matWorldViewProj;
		
		float4 vecFog;
		float4 vecTime;
		float4 vecColor;
		float4 vecViewDir;
		float4 vecViewPos;
		float4 vecSunDir;
		float4 vecSunColor;
		float4 vecLight;
		float4 vecLightPos[8];
		float4 vecLightColor[8];
		
		float4 vecSkill41;
		float4 vecSkill45;
		
		
		//Textures
		texture entSkin1;
		
		
		//Sampler
		sampler colorMap = sampler_state
		{
			Texture = (entSkin1);
			AddressU = Wrap;
			AddressV = Wrap;
			
			MagFilter = Linear;
			MinFilter = Linear;
			MipFilter = Linear;
		};
		

		//Structs
		struct VS_TO_PS // Output to the pixelshader
		{
			float4 Pos	: POSITION;
			float  Fog	: FOG;
			
			float2 Tex0	: TEXCOORD0;
			float3 WPos	: TEXCOORD1;
			float3 Norm	: TEXCOORD2;
			
			float3 ViewDir : TEXCOORD3;
		};
		
		
		//Vertexshader
		VS_TO_PS VS_v0(	float4 inPos		: POSITION,
								float3 inNormal	: NORMAL,
								float2 inTex0		: TEXCOORD0	)
		{
			VS_TO_PS Out;
			
			//Do transformations
			Out.Pos = mul(inPos,matWorldViewProj);
			
			//Pass the Texcoords
			Out.Tex0 = inTex0;
			
			//Vertexposition in worldspace
			Out.WPos = mul(inPos,matWorld);
			
			//Vertexnormal
			Out.Norm = normalize(mul(inNormal,matWorld));
			
			//Direction to the view
			Out.ViewDir = normalize(Out.WPos-vecViewPos);
			
			//Fog
			Out.Fog = 1 - (distance(Out.WPos,vecViewPos)-vecFog.x) * vecFog.z;
			
			return Out;
		}
		
		
		//Pixelshader
		float4 PS_p0( VS_TO_PS In ) : COLOR
		{
			float3 LightDir;
			float3 Diffuse = vecLight.rgb+saturate(dot(-vecSunDir,In.Norm))*vecSunColor;
			float attenuation = 0;
			
			for(int i = 0; i < 2; i++)
			{
				LightDir = vecLightPos[i]-In.WPos;
				attenuation = saturate(1.0f - length(LightDir/vecLightPos[i].w));
				Diffuse += saturate(dot(normalize(LightDir),In.Norm))*vecLightColor[i]*attenuation;
			}
			
			float Glow = saturate(dot(In.ViewDir,In.Norm));
			Glow = (pow(Glow*0.5+1,vecSkill1.x)-1)*vecSkill41.y;
			
			float4 Color = tex2D(colorMap,In.Tex0);
			Color.rgb *= Diffuse;
			
			Color += vecSkill45*Glow;	//lerp(Color,vecSkill5,Glow);
			Color.a = 1.0;
			
			return Color;
		}
		
		
		//////////////////////////////////////////////////////////////////
		technique Lighting
		{
			pass one
			{
				AlphaTestEnable = True;
				zWriteEnable = true;
				
				VertexShader = compile vs_1_1 VS_v0();
				PixelShader = compile ps_2_0 PS_p0();
			}
		}
		
		technique fallback { pass one { } }
	";
}


//initializes and activates the effect
void OS_Glow_init(ENTITY* Ent, var BGLWidth, var BGLStrength, var BGLColRed, var BGLColGreen, var BGLColBlue)
{
	if(d3d_shaderversion >= 2020)
	{
		Ent.material = OS_Glow_ps20_mat;
		
		Ent.skill41 = floatv(BGLWidth);
		Ent.skill42 = floatv(BGLStrength);
		
		Ent.skill45 = floatd(BGLColRed,255);
		Ent.skill46 = floatd(BGLColGreen,255);
		Ent.skill47 = floatd(BGLColBlue,255);
	}
}

//function to change the material parameters after initializing
void OS_Glow_SetValues(ENTITY* Ent, var BGLWidth, var BGLStrength, var BGLColRed, var BGLColGreen, var BGLColBlue)
{
	Ent.skill41 = floatv(BGLWidth);
	Ent.skill42 = floatv(BGLStrength);
	
	Ent.skill45 = floatd(BGLColRed,255);
	Ent.skill46 = floatd(BGLColGreen,255);
	Ent.skill47 = floatd(BGLColBlue,255);
}

#endif