//////////////////////////////////////////////////
///////////////////Watershader////////////////////
//////////////////////////////////////////////////
////02.04.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////

#ifndef OS_Water
#define OS_Water


VIEW* OS_Water_Reflect_view;
VIEW* OS_Water_Refract_view;

BMAP* OS_Water_Reflect_bmap;
BMAP* OS_Water_Refract_bmap;


//PS 2.0 Watershader with reflection and refraction
MATERIAL* OS_Water_ps20_mat =
{
	effect =
	"
		//Variables
		float4x4 matWorld;
		float4x4 matView;
		float4x4 matProj;
		float4x4 matWorldViewProj;
		
		float4 vecFog;
		float4 vecViewPos;
		float4 vecTime;
		float4 vecColor;
		float4 vecViewDir;
		
		float4 vecSkill1; // Scale, Wind_X, Wind_Y, Ripple
		
		
		//Textures
		texture mtlSkin1;
		texture mtlSkin2;
		texture mtlSkin3;
		texture mtlSkin4;
		
		
		//Sampler
		sampler reflectionMap = sampler_state
		{
			Texture = (mtlSkin1);
			AddressU = Clamp;
			AddressV = Clamp;
			
			MagFilter = Linear;
			MinFilter = Point;
			MipFilter = None;
		};
		
		sampler refractionMap = sampler_state
		{
			Texture = (mtlSkin2);
			AddressU = Clamp;
			AddressV = Clamp;
			
			MagFilter = Linear;
			MinFilter = Point;
			MipFilter = None;
		};
		
		sampler bumpMap = sampler_state { Texture = (mtlSkin3); };
		sampler blendMap = sampler_state { Texture = (mtlSkin4); };
		
		
		//Structs
		struct VS_TO_PS // Output to the pixelshader
		{
			float4 Pos	: POSITION;
			float  Fog	: FOG;
			
			float4 Tex0	: TEXCOORD0;	//Bumpmap, Blendmap
			float3 Tex1	: TEXCOORD1;	//Reflection and Refraction
//			float fresnel : TEXCOORD2;
		};
		
		
		//Vertexshader
		VS_TO_PS VS_v0(	float4 inPos		: POSITION,
								float3 inNormal	: NORMAL,
								float2 inTex0		: TEXCOORD0	)
		{
			VS_TO_PS Out;
			
			//do the transformations
			Out.Pos = mul(inPos,matWorldViewProj);
			
			//Fog
			float3 P = mul(inPos,matWorld);
			Out.Fog = 1 - (distance(P,vecViewPos)-vecFog.x) * vecFog.z;
			
			// bump coordinates
			Out.Tex0.xy = (inTex0 + vecSkill1.yz * vecTime.w) * vecSkill1.x;
			Out.Tex0.zw = inTex0;
			
			//reflection and refraction coordinates
			Out.Tex1 = Out.Pos.xyw;
			
			//fresnel
//			Out.fresnel = saturate(1-dot(float3(0,1,0),normalize(-vecViewDir.xyz))*(1-distance(vecViewPos,P)*0.01));
			
			return Out;
		}
		
		
		//Pixelshader
		float4 PS_p0( VS_TO_PS In ) : COLOR 
		{
			//get the bumpmap
			float4 Bump = tex2D(bumpMap, In.Tex0.xy) * 2 - 1;
			float reflrefrblend = tex2D(blendMap,In.Tex0.zw).r;
			
			//get the projection coords
			float2 temp = In.Tex1.xy/In.Tex1.z;
			float2 CoordsRefl = 0.5*(1.0+temp)+Bump.xy*vecSkill1.w*reflrefrblend;
			
			//get the reflectionmap and the refractionmap
			float4 reflection = tex2D(reflectionMap, CoordsRefl);
			float4 refraction = tex2D(refractionMap, CoordsRefl);
			
			float4 reflrefr = lerp(refraction,reflection,reflrefrblend);
			reflrefr.a = 1.0;
			
			// Output
			return reflrefr;
		}
		
		//////////////////////////////////////////////////////////////////
		technique water_env
		{
			pass one
			{
				AlphaBlendEnable = False;
				cullmode = none;
				
				VertexShader = compile vs_1_1 VS_v0();
				PixelShader = compile ps_2_0 PS_p0();
			}
		}
		
		technique fallback { pass one { } }
	";
}

//ffp reflecting water
MATERIAL* OS_Water_ffp_mat =
{
	effect =
	"
		//Variables
		float4x4 matMtl;
		
		//Textures
		texture mtlSkin1;
		texture mtlSkin2;
		
		//Technique
		technique Water
		{
			pass p0
			{
				AlphaBlendEnable = True;  
				Zenable = True;
				ZwriteEnable = True;
				
				BlendOp = Add;
				Lighting = True;
				
				Texture[0] = <mtlSkin2>;
				
				magFilter[0] = linear;
				minFilter[0] = linear;
				mipFilter[0] = linear;
				
				COLOROP[0] = BumpEnvMap;
				COLORARG1[0] = Texture;
				COLORARG2[0] = Current;
				
				TexCoordIndex[0] = 0;
				TextureTransformFlags[0] = Count2;
				TextureTransform[0] = <matMtl>;
				
				Texture[1] = <mtlSkin1>;
				
				magFilter[1] = Linear;
				minFilter[1] = Linear;
				mipFilter[1] = Linear;
				
				AddressU[1] = Clamp;
				AddressV[1] = Clamp;
				
				ColorOp[1] = Modulate;
				ColorArg1[1] = Texture;
				ColorArg2[1] = Current;
				
				texcoordindex[1] = cameraspaceposition | 0;
				TextureTransformFlags[1] = count3 | projected;
				TextureTransform[1] =
				{
					0.8,	0.0,	0.0,	0.0,
					0.0,	1.15,	0.0,	0.0,
					0.0,	0.0,	1.0,	0.0,
					0.0,	0.0,	0.0,	1.0
				};
			}
		}
	";
}

//creates and handles the reflection view
void OS_Water_ReflectView_init(var SizeX, var SizeY)
{
	if(OS_Water_Reflect_view == NULL)
	{
		OS_Water_Reflect_view = view_create(-1);
	}
	if(OS_Water_Reflect_bmap == NULL)
	{
		OS_Water_Reflect_bmap = bmap_createblack(SizeX,SizeY,24);
	}
	
	OS_Water_Reflect_view.size_x = bmap_width(OS_Water_Reflect_bmap);
	OS_Water_Reflect_view.size_y = bmap_width(OS_Water_Reflect_bmap);
	OS_Water_Reflect_view.aspect = screen_size.x/screen_size.y;
	
	vec_set(OS_Water_Reflect_view.pnormal_x,vector(0,0,1));
	
	set(OS_Water_Reflect_view,VISIBLE|NOSHADOW|NOFLAG1|NOPARTICLE);
	
	OS_Water_Reflect_view.bmap = OS_Water_Reflect_bmap;
	
	proc_mode = PROC_LATE;
	
	while(is(OS_Water_Reflect_view,VISIBLE))
	{
		OS_Water_Reflect_view.fog_start = camera.fog_start;
		OS_Water_Reflect_view.fog_end = camera.fog_end;
		OS_Water_Reflect_view.clip_far = camera.clip_far*0.5;
		OS_Water_Reflect_view.arc = camera.arc;
		OS_Water_Reflect_view.ambient = camera.ambient;
		
		OS_Water_Reflect_view.x 	   = camera.x;
		OS_Water_Reflect_view.y 	   = camera.y;
		OS_Water_Reflect_view.z 	   = 2*OS_Water_Reflect_view.portal_z-camera.z;
		
		OS_Water_Reflect_view.pan    = camera.pan;
		OS_Water_Reflect_view.tilt   = -camera.tilt;
		OS_Water_Reflect_view.roll   = -camera.roll;
		
		wait(1);
	}
}

//creates and handles the refraction view
void OS_Water_RefractView_init(var SizeX, var SizeY)
{
	if(OS_Water_Refract_view == NULL)
	{
		OS_Water_Refract_view = view_create(-1);
	}
	if(OS_Water_Refract_bmap == NULL)
	{
		OS_Water_Refract_bmap = bmap_createblack(SizeX,SizeY,24);
	}
	
	OS_Water_Refract_view.size_x = bmap_width(OS_Water_Refract_bmap);
	OS_Water_Refract_view.size_y = bmap_width(OS_Water_Refract_bmap);
	OS_Water_Refract_view.aspect = -screen_size.x/screen_size.y;
	
	vec_set(OS_Water_Refract_view.pnormal_x,vector(0,0,-1));
	
	set(OS_Water_Refract_view,VISIBLE|NOSHADOW|NOFLAG1|NOPARTICLE);
	
	OS_Water_Refract_view.bmap = OS_Water_Refract_bmap;
	
	proc_mode = PROC_LATE;
	
	while(is(OS_Water_Refract_view,VISIBLE))
	{
		OS_Water_Refract_view.fog_start = camera.fog_start;
		OS_Water_Refract_view.fog_end = camera.fog_end;
		OS_Water_Refract_view.clip_far = camera.clip_far;
		OS_Water_Refract_view.arc = camera.arc;
		OS_Water_Refract_view.ambient = camera.ambient;
		
		vec_set(OS_Water_Refract_view.x,camera.x);
		vec_set(OS_Water_Refract_view.pan,camera.pan);
		
		wait(1);
	}
}

//assigns the shader to the entity
void OS_Water_init(ENTITY* Ent, BMAP* bumpMap, BMAP* blendMap, var BumpTilefactor, var BumpHeight, var WindX, var WindY, var ClipOffset, var ReflectRes, var RefractRes)
{
	if(d3d_shaderversion >= 2020)
	{
		MATERIAL* Temp_Mat = mtl_create();
		
		Temp_Mat.effect = OS_Water_ps20_mat.effect;
		Ent.material = Temp_Mat;
		
		OS_Water_ReflectView_init(ReflectRes,ReflectRes);
		OS_Water_RefractView_init(RefractRes,RefractRes);
		
		set(Ent,FLAG1);
		OS_Water_Reflect_view.portal_z = Ent.z-ClipOffset;
		OS_Water_Refract_view.portal_z = Ent.z+ClipOffset;
		
		bmap_to_mipmap(bumpMap);
		bmap_to_mipmap(blendMap);

		Temp_Mat.skin1 = OS_Water_Reflect_bmap;
		Temp_Mat.skin2 = OS_Water_Refract_bmap;
		Temp_Mat.skin3 = bumpMap;
		Temp_Mat.skin4 = blendMap;
		
		Temp_Mat.skill1 = floatv(BumpTilefactor);
		Temp_Mat.skill2 = floatd(WindX,10000);
		Temp_Mat.skill3 = floatd(WindY,10000);
		Temp_Mat.skill4 = floatd(BumpHeight,10);
	}else
	{
		MATERIAL* Temp_Mat = mtl_create();
		Temp_Mat.effect = OS_Water_ffp_mat.effect;
		Ent.material = Temp_Mat;
		OS_Water_ReflectView_init(ReflectRes,ReflectRes);
		
		OS_Water_Reflect_view.portal_z = Ent.z-ClipOffset;
		
		bmap_to_mipmap(bumpMap);
		
		Temp_Mat.skin1 = OS_Water_Reflect_bmap;
		Temp_Mat.skin2 = bumpMap;
		
		mat_identity(Temp_Mat.matrix);
		Temp_Mat.matrix11 = BumpTilefactor;
		Temp_Mat.matrix22 = BumpTilefactor;
		
		float Wind[2] = {0,0};
		while((my != NULL) && (Temp_Mat == Ent.material))
		{
			Wind[0] += time_step*0.001*WindX;
			Wind[1] += time_step*0.001*WindY;
			
			Temp_Mat.matrix31 = Wind[0];
			Temp_Mat.matrix32 = Wind[1];
			
			wait(1);
		}
	}
}

#endif