//////////////////////////////////////////////////
///////////////Postprocessing Blood///////////////
//////////////////////////////////////////////////
////01.04.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////
#define PP_Blood
#define PP_Blood_Max 10


//struct to store the needed parameters
typedef struct
{
	BMAP* bmap_color;
	BMAP* bmap_normal;
	
	float pos_x;
	float pos_y;
	float scale_x;
	float scale_y;
	float alpha;
	
	VIEW* View;
}PP_BLOOD_VALUES;

//array of the struct for up to PP_Blood_Max bloodsplashes
PP_BLOOD_VALUES PP_Blood_Values[PP_Blood_Max];


//materialevent to set the values for the splash
void PP_Blood_mat_event()
{
	mtl.skin1 = PP_Blood_Values[render_view.layer].bmap_color;
	mtl.skin2 = PP_Blood_Values[render_view.layer].bmap_normal;
	
	mtl.skill1 = PP_Blood_Values[render_view.layer].pos_x;
	mtl.skill2 = PP_Blood_Values[render_view.layer].pos_y;
	mtl.skill3 = PP_Blood_Values[render_view.layer].scale_x;
	mtl.skill4 = PP_Blood_Values[render_view.layer].scale_y;
	mtl.skill5 = PP_Blood_Values[render_view.layer].alpha;
}

//PS 2.0 blood material
MATERIAL* PP_Blood_mat =
{
	event = PP_Blood_mat_event;
	flags = ENABLE_VIEW;
	
	effect =
	"
		// Variables
		float4 vecSkill1;
		float4 vecSkill5;
		float4 vecViewDir;
		float4 vecSunDir;
		
		// Textures
		texture TargetMap;
		texture mtlSkin1;
		texture mtlSkin2;
		
		sampler postTex = sampler_state
		{
		   texture = <TargetMap>;
		   MinFilter = linear;
		   MagFilter = linear;
		   MipFilter = linear;
		   AddressU = Clamp;
		   AddressV = Clamp;
		};
		
		sampler colorTex = sampler_state
		{
		   texture = <mtlSkin1>;
		   MinFilter = linear; 
		   MagFilter = linear; 
		   MipFilter = linear; 
		   AddressU = Clamp;
		   AddressV = Clamp;
		};
		
		sampler bumpTex = sampler_state
		{
		   texture = <mtlSkin2>;
		   MinFilter = linear; 
		   MagFilter = linear; 
		   MipFilter = linear; 
		   AddressU = Clamp;
		   AddressV = Clamp;
		};
		
		// Pixelshader 
		float4 PS_Blood(float2 TexCoords : TEXCOORD0) : COLOR
		{
			float2 Temp_TexCoords = (TexCoords-vecSkill1.xy)*vecSkill1.zw+0.5;
			float3 BumpColor = tex2D(bumpTex,Temp_TexCoords).rgb;
			float4 TexColor = tex2D(colorTex,Temp_TexCoords);
			
			Temp_TexCoords = TexCoords+BumpColor.rg * 0.05f*vecSkill5.x;
			float3 ResultColor = lerp(tex2D(postTex,Temp_TexCoords).rgb,TexColor.rgb,TexColor.a*vecSkill5.x);
		   
		   return float4(ResultColor,1.0);
		}
		
		//
		technique tech_00
		{
		   pass pass_00
		   {
		      PixelShader = compile ps_2_0 PS_Blood();
		   }
		}
	";
}

//adjust the splashs values
void PP_Blood_SetValues(BMAP* bmapColor, BMAP* bmapNormal, int index, var xPosition, var yPosition, var xScale, var yScale, var Alpha)
{
	if(d3d_shaderversion >= 2020)
	{
		if(bmapColor){PP_Blood_Values[index].bmap_color = bmapColor;}
		if(bmapNormal){PP_Blood_Values[index].bmap_normal = bmapNormal;}
		
		PP_Blood_Values[index].pos_x = floatv(xPosition);	//between 0 and screen_size.x
		PP_Blood_Values[index].pos_y = floatv(yPosition);	//between 0 and screen_size.y
		PP_Blood_Values[index].scale_x = floatv(xScale); 	//about 1
		PP_Blood_Values[index].scale_y = floatv(yScale); 	//about 1
		PP_Blood_Values[index].alpha = floatv(Alpha); 		//between 0 and 1
	}
}

//adds one bloodsplash
void PP_Blood_init(VIEW* View, BMAP* bmapColor, BMAP* bmapNormal, int index, var xPosition, var yPosition, var xScale, var yScale, var Alpha)
{
	PP_Blood_SetValues(index,bmapColor,bmapNormal,xPosition,yPosition,xScale,yScale,Alpha);
	PP_Blood_Values[index].View = PP_Add(PP_Blood_mat,View,NULL);
	layer_sort(PP_Blood_Values[index].View,index);
}

//removes the splash with the given index
void PP_Blood_remove(VIEW* View, int index)
{
	PP_Remove(PP_Blood_mat,View,PP_Blood_Values[index].View);
}