//////////////////////////////////////////////////
///////////Postprocessing Motion Blur/////////////
//////////////////////////////////////////////////
////11.06.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////
#define PP_MotionBlur


//transforms the colors with the material matrix
MATERIAL* PP_MotionBlur_mat =
{
	effect =
	"
		//Texture
		texture mtlSkin1;
		texture mtlSkin2;
		texture mtlSkin3;
		texture mtlSkin4;
		
		//Samplers
		sampler2D ColorSampler1 = sampler_state { texture = <mtlSkin1>;};
		sampler2D ColorSampler2 = sampler_state { texture = <mtlSkin2>;};
		sampler2D ColorSampler3 = sampler_state { texture = <mtlSkin3>;};
		sampler2D ColorSampler4 = sampler_state { texture = <mtlSkin4>;};
		
		//Pixelshader
		float4 PS( float2 texCoord : TEXCOORD0 ) : COLOR0 
		{
			float4 Color = tex2D(ColorSampler1,texCoord)*0.4;
			Color += tex2D(ColorSampler2,texCoord)*0.3;
			Color += tex2D(ColorSampler3,texCoord)*0.2;
			Color += tex2D(ColorSampler4,texCoord)*0.1;
			
			return float4(Color.rgb,1.0);
		}
		
		technique Tech1
		{
			pass one
			{
				PixelShader = compile ps_2_0 PS();
			}
		}
	";
}


//starts the effect
void PP_MotionBlur_init(VIEW* View)
{
	if(d3d_shaderversion >= 2020)
	{
		BMAP* bmaps[4];
		bmaps[0] = bmap_createblack(screen_size.x,screen_size.y,24);
		bmaps[1] = bmap_createblack(screen_size.x,screen_size.y,24);
		bmaps[2] = bmap_createblack(screen_size.x,screen_size.y,24);
		bmaps[3] = bmap_createblack(screen_size.x,screen_size.y,24);
		
		View = PP_LastView(View);
		
		//add the effect as a next stage
		PP_Add(PP_MotionBlur_mat,View,NULL);
		
		int i = 0;
		while(1)
		{
			View.bmap = bmaps[i];
			
			PP_MotionBlur_mat.skin1 = bmaps[i%4];
			PP_MotionBlur_mat.skin2 = bmaps[(i+1)%4];
			PP_MotionBlur_mat.skin3 = bmaps[(i+2)%4];
			PP_MotionBlur_mat.skin4 = bmaps[(i+3)%4];
			
			i += 1;
			i %= 4;
			
			wait(-0.05);
		}
	}
}

//removes ColorFiler
void PP_MotionBlur_remove(VIEW* View)
{
	//remove the stage
	PP_Remove(PP_MotionBlur_mat,View,NULL);
}