//////////////////////////////////////////////////
////////////Postprocessing Radial Blur////////////
//////////////////////////////////////////////////
////07.04.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////
#define PP_RadialBlur


//a radial blur shader
MATERIAL* PP_RadialBlur_mat =
{
	effect =
	"
		#define NUM_SAMPLES 20
		
		//Variables
		float4 vecViewPort;
		float4 vecSkill1;
		
		//Texture
		texture TargetMap;
		sampler2D ColorSampler = sampler_state { texture = <TargetMap>;};
		
		//Pixelshader
		float4 PS( float2 texCoord : TEXCOORD0 ) : COLOR0 
		{
			float2 deltaTexCoord = (texCoord - vecSkill1.xy);
			deltaTexCoord *= 1.0f / NUM_SAMPLES * vecSkill1.z;
			
			float4 Color = 0;
			
			for(int i = 0; i < NUM_SAMPLES; i++)
			{
				texCoord -= deltaTexCoord;
				Color += tex2D(ColorSampler, texCoord);
			}
			
			Color /= NUM_SAMPLES;
			Color.a = 1.0;
			return Color;
		}
		
		technique Tech1
		{
			pass one
			{
				PixelShader = compile ps_2_0 PS();
			}
		}
	";
}

//change the values of the shader
void PP_RadialBlur_SetValues(var xPosition, var yPosition, var Density)
{
	PP_RadialBlur_mat.skill1 = floatv(xPosition);	//between 0 and 1
	PP_RadialBlur_mat.skill2 = floatv(yPosition);	//between 0 and 1
	PP_RadialBlur_mat.skill3 = floatv(Density);		//small, around 0.25
}

//starts the radial blur
void PP_RadialBlur_init(VIEW* View, var xPosition, var yPosition, var Density)
{
	if(d3d_shaderversion >= 2020)
	{
		PP_RadialBlur_SetValues(xPosition,yPosition,Density);
		PP_Add(PP_RadialBlur_mat,View,NULL);
	}
}

//removes the effect
void PP_RadialBlur_remove(VIEW* View)
{
	//remove the stage
	PP_Remove(PP_RadialBlur_mat,View,NULL);
}