//////////////////////////////////////////////////
////////////////Renders a Depthmap////////////////
//////////////////////////////////////////////////
////08.03.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////
#define PP_Shared_Depth

#define PP_Shared_Depth_TexType 888


VIEW* PP_Shared_Depth_orig_view;


//material which renders a depthmap
MATERIAL* PP_Shared_Depth_mat =
{
	effect =
	"
		#define PP_Shared_Depth_Precision 10000
		
		float4x4 matWorldViewProj;
		
		float4 vecSkill41;
		
		texture entSkin1;
		sampler smpSkin = sampler_state { Texture = <entSkin1>; };
		
		struct VS_OUT
		{
			float4 position : POSITION;
			float  depth    : TEXCOORD0;
			float2 texcoords : TEXCOORD1;
		};
		
		VS_OUT mainVS (float4 position : POSITION, float2 texcoords : TEXCOORD0)
		{
			VS_OUT Out;
			Out.position = mul(position, matWorldViewProj);
			Out.depth = Out.position.z;
			
			if(vecSkill41.x == 1){Out.depth = 0;}
			Out.texcoords = texcoords;
			return Out;
		}
		
		float4 mainPS(float depth : TEXCOORD0, float2 texcoords : TEXCOORD1) :COLOR
		{
			float4 Color = 1.0;
			depth = saturate(depth/PP_Shared_Depth_Precision);
			
			const float max24int = 256*256*256-1;
		   Color.rgb = floor(depth*float3(max24int/(256*256),max24int/256,max24int))/255.0;
		   Color.b -= Color.g * 256.0;
		   Color.g -= Color.r * 256.0;
			
			Color.a = step(0.5,tex2D(smpSkin,texcoords).a);
			
			return(Color);
		}
		
		technique PPE
		{
			pass p1
			{
				alphaBlendEnable = false;
				alphaTestEnable = true;
				zWriteEnable = true;
				
				VertexShader = compile vs_1_1 mainVS();
				PixelShader = compile ps_2_0 mainPS();
			}
		}
	";
}


//initializes the depthmap rendering
void PP_Shared_Depth_init(VIEW* View)
{
	if(PP_Shared_Depth_orig_view == NULL)
	{
		PP_Shared_Depth_orig_view = view_create(-1);
	}
	set(PP_Shared_Depth_orig_view,VISIBLE|NOPARTICLE);
	PP_Shared_Depth_orig_view.size_x = screen_size.x;
	PP_Shared_Depth_orig_view.size_y = screen_size.y;
	PP_Shared_Depth_orig_view.material = PP_Shared_Depth_mat;
	
	PP_Shared_Depth_orig_view.bmap = bmap_createblack(screen_size.x,screen_size.y,PP_Shared_Depth_TexType);
	
	proc_mode = PROC_LATE;
	
	while(View != NULL && PP_Shared_Depth_orig_view != NULL)
	{
		//update the views position
		vec_set(PP_Shared_Depth_orig_view.x,View.x);
		vec_set(PP_Shared_Depth_orig_view.pan,View.pan);
		PP_Shared_Depth_orig_view.arc = View.arc;
		PP_Shared_Depth_orig_view.aspect = View.aspect;
		
		wait(1);
	}
	
	reset(PP_Shared_Depth_orig_view,VISIBLE);
}

//makes it fit to the new resolution
void PP_Shared_Depth_Video_Switch()
{
	if(PP_Shared_Depth_orig_view != NULL)
	{
		ptr_remove(PP_Shared_Depth_orig_view.bmap);
		
		PP_Shared_Depth_orig_view.size_x = screen_size.x;
		PP_Shared_Depth_orig_view.size_y = screen_size.y;
		
		PP_Shared_Depth_orig_view.bmap = bmap_createblack(screen_size.x,screen_size.y,PP_Shared_Depth_TexType);
	}
}